/*
 * Copyright (c) 2020, Ford Motor Company
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided with the
 * distribution.
 *
 * Neither the name of the Ford Motor Company nor the names of its contributors
 * may be used to endorse or promote products derived from this software
 * without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef SRC_COMPONENTS_APPLICATION_MANAGER_INCLUDE_APPLICATION_MANAGER_RESUMPTION_PENDING_RESUMPTION_HANDLER_H
#define SRC_COMPONENTS_APPLICATION_MANAGER_INCLUDE_APPLICATION_MANAGER_RESUMPTION_PENDING_RESUMPTION_HANDLER_H
#include "application_manager/app_extension.h"
#include "application_manager/application_manager.h"
#include "application_manager/event_engine/event_observer.h"

namespace resumption {

namespace app_mngr = application_manager;

/**
 * @brief The PendingResumptionHandler class responsibility to avoid
 * duplication of subscription requests to HMI if multiple applications
 * are registering
 */
class PendingResumptionHandler
    : public application_manager::event_engine::EventObserver {
 public:
  PendingResumptionHandler(app_mngr::ApplicationManager& application_manager);

  virtual ~PendingResumptionHandler() {}

  /**
   * @brief HandleResumptionSubscriptionRequest handles all subscriptions
   * requests, which might be send to HMI under certain conditions
   * @param extension reference to corresponding application extension
   * @param subscriber callback for subscribing
   * @param app reference to application owner
   */
  virtual void HandleResumptionSubscriptionRequest(
      app_mngr::AppExtension& extension,
      application_manager::Application& app) = 0;

  /**
   * @brief OnResumptionRevert is a callback to be called in the case, when
   * resumption is failed for current application.
   */
  virtual void OnResumptionRevert() = 0;

  resumption::ResumptionDataProcessor& resumption_data_processor();

 protected:
  /**
   * @brief MakeResumptionRequest creates resumption request
   * @param corr_id correlation ID
   * @param function_id function ID
   * @param message Message to HMI
   * @return instance of ResumptionRequest class
   */
  ResumptionRequest MakeResumptionRequest(
      const uint32_t corr_id,
      const hmi_apis::FunctionID::eType function_id,
      const smart_objects::SmartObject& message);

  app_mngr::ApplicationManager& application_manager_;
};
}  //  namespace resumption

#endif  // SRC_COMPONENTS_APPLICATION_MANAGER_INCLUDE_APPLICATION_MANAGER_RESUMPTION_PENDING_RESUMPTION_HANDLER_H
