/*
 * Copyright (c) 2016, Ford Motor Company
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided with the
 * distribution.
 *
 * Neither the name of the Ford Motor Company nor the names of its contributors
 * may be used to endorse or promote products derived from this software
 * without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
#include "application_manager/app_launch/app_launch_sql_queries.h"

namespace app_launch {

const std::string kCreateSchema =
    "CREATE TABLE IF NOT EXISTS `app_launch`( "
    "  `deviceMac` TEXT, "
    "  `appID` TEXT collate nocase,"
    "  `bundleID` TEXT,"
    "  `last_session` DATETIME, "
    " PRIMARY KEY(`deviceMac`, `appID`, `bundleID`)"
    "  ); ";

const std::string kDropSchema = "DROP TABLE IF EXISTS `app_launch`; ";

const std::string kAddApplicationData =
    "INSERT INTO `app_launch`"
    "(`deviceMac`, `appID`, `bundleID`, `last_session`)"
    "VALUES "
    "(?, ?, ?, STRFTIME('%Y-%m-%d %H:%M:%f', 'NOW'));";

const std::string kFindApplicationData =
    " SELECT COUNT(*)"
    "FROM `app_launch`"
    "WHERE `deviceMac` = ? AND `appID` = ? AND `bundleID` = ?;";

const std::string kDeleteOldestAppData =
    "DELETE FROM `app_launch`"
    "WHERE `last_session` IN ("
    "SELECT  MIN(`last_session`)"
    "FROM `app_launch`);";

const std::string kGetNumberOfApplicationData =
    "SELECT COUNT (*)"
    "FROM `app_launch` ;";

const std::string kGetApplicationDataByDevID =
    "SELECT *"
    "FROM `app_launch`"
    "WHERE `deviceMac` = ?;";

const std::string kRefreshApplicationDataSessionTime =
    "UPDATE `app_launch`"
    "SET `last_session` = STRFTIME('%Y-%m-%d %H:%M:%f', 'NOW')"
    "WHERE `deviceMac` = ? AND appID = ? AND bundleID = ?;";

}  // namespace app_launch
