/*
 * Copyright (c) 2019, Ford Motor Company
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided with the
 * distribution.
 *
 * Neither the name of the Ford Motor Company nor the names of its contributors
 * may be used to endorse or promote products derived from this software
 * without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef SRC_COMPONENTS_INCLUDE_RESUMPTION_LAST_STATE_H_
#define SRC_COMPONENTS_INCLUDE_RESUMPTION_LAST_STATE_H_

#include "json/json.h"
#include "utils/macro.h"

namespace resumption {

class LastState {
 public:
  /**
   * @brief Destructor
   */
  virtual ~LastState() {}

  /**
   * @brief SaveToFileSystem
   * Saving dictionary to filesystem
   */
  virtual void SaveToFileSystem() = 0;

  /**
   * @brief RemoveFromFileSystem
   * Remove dictionary from filesystem
   */
  virtual void RemoveFromFileSystem() = 0;

  /**
   * @brief dictionary Gets internal dictionary
   * @return Copy of internal dictionary json value
   */
  virtual Json::Value dictionary() const = 0;

  /**
   * @brief set_dictionary sets internal dictionary
   * @param dictionary New dictionary json value to be set
   */
  virtual void set_dictionary(const Json::Value& dictionary) = 0;
};

}  // namespace resumption

#endif  // SRC_COMPONENTS_INCLUDE_RESUMPTION_LAST_STATE_H_
